<?php

namespace App;

use DateTime;
use Nette\DI\Container;
use Nette\Mail\Message,
	Nette\Diagnostics\Debugger,
	Nette\Utils\Html,
	Murdej\Duration;

class Glob {
	static function emailMessage($templateName, $data = array(), $layoutName = "default") {
		$data += array(
			'from' => Utils::config()->defaultEmailFrom
		);
		$mail = new Message;
		$mail->setFrom($data['from']);
		$mail->setMailer(new QueueMailer);

		$data['mail'] = $mail;
		$layout = new \Nette\Templating\FileTemplate(APP_DIR.'/emailTemplates/'.$layoutName.'Layout.latte');
		$layout->registerFilter(new \Nette\Latte\Engine);
		self::setupHeplers($layout);
		
		$template = new \Nette\Templating\FileTemplate(APP_DIR.'/emailTemplates/'.$templateName.'.latte');
		$template->registerFilter(new \Nette\Latte\Engine);
		self::setupHeplers($template);
		
		foreach($data as $key => $value) {
			$layout->{$key} = $value;
			$template->{$key} = $value;
		}
		
		$layout->content = $template;
		
		$mail->setHtmlBody($layout->__tostring());
		
		return $mail;
	}
	
	static function logOperation($operation, $oldObject = null, $newObject = null) {
		global $container;
		$container->operationLogs->add($operation, $oldObject, $newObject);
	}

	static $safePasswordRE = '\s*.{6,}\s*';
	
	static $codeRE = '\s*[0-9]+\s*';
	
	static $safePasswordDescription = 'Heslo musí mít aspoň 6 znaků';
	
	static function context() : Container {
		global $container;
		return $container;
	}
	
	static $defaultDateFormat = 'd.m.Y';
	
	static $defaultDateTimeFormat = "d.m.Y H:i";
	
	static function formatDateLong($ut) {
		return self::formatDate($ut, 'long');
	}

	static function formatDateTimeSmart($ut) {
		return
			(date('d.m.Y') == date('d.m.Y', $ut)
			? 'dnes' : date('d.m.Y', $ut))
			. ' ' . date('H:i', $ut);
	}
	
	static function formatDateTime($ut) {
		return $ut 
			? (is_int($ut)
				? date('d.m.Y H:i:s', $ut)
				: $ut->format('d.m.Y H:i:s')) 
			: '';
	}
	
	static function formatDateApproximate($ut) {
		return self::formatDate($ut, 'approximate');
	}
	
	static function formatDate($ut, $format = 'normal') {
		
		if ($ut) {
			if (($format == 'normal') || ($format == 'long')) {
				$ret = date(self::$defaultDateFormat, $ut);
			} else {
				$ret = '';
			}
			if (($format == 'long') || ($format == 'approximate')) {
				$months = array('Leden','Únor','Březen','Duben','Květen','Červen', 'Červenec','Srpen','Září','Říjen','Listopad','Prosinec');
				$ret .= ($ret ? ' (' : '')
					.$months[((int)date('n', $ut)) - 1].' '.date('Y', $ut)
					.($ret ? ')' : '');
			}
		} else {
			$ret = '-';
		}
		return $ret;
	}
	
	static function formatDateAuto($ut, $duration) {
		return self::formatDate($ut, 
			$duration > (24*3600*30)
			? 'approximate'
			: 'normal'
		);
	}

	static function formatDuration($ut) {
		return $ut ? Duration::toLongString($ut, '< 1d') : '-';
	}

	static function setupHeplers(&$template) {
		//$texy = self::texy();
		// $template->registerHelper("texy", array($texy, "process"));
		$template->addFilter("utDate", array('\App\Glob', 'formatDate'));
		$template->addFilter("price", array('\App\Glob', 'formatPrice'));
		$template->addFilter("priceSmallCurency", array('\App\Glob', 'formatPriceSmallCurency'));
		$template->addFilter("utDateTime", array('\App\Glob', 'formatDateTime'));
		$template->addFilter("utDateTimeSmart", array('\App\Glob', 'formatDateTimeSmart'));
		$template->addFilter("duration", array('\App\Glob', 'formatDuration'));
		$template->addFilter("utDateLong", array('\App\Glob', 'formatDateLong'));
		$template->addFilter("numberZB", array('\App\Glob', 'numberZB'));
	}
	
	static $_texy = null;
	
	static function numberZB($num)
	{
		// $tmp = explode('.', $num);
		return $num ? $num : "";
	}

	static function texy() {
		if (!self::$_texy) {
			self::$_texy = new \Texy;
			self::$_texy->allowed['html/tag'] = TEXY_ALL;
		}
		return self::$_texy;
	}
	
	static function utRound($ut) {
		$dt = new \DateTime(date('Y-m-d', $ut));
		return $dt->getTimestamp();
	}
	
	static function varbar($var) {
		\Tracy\Debugger::barDump($var);
	}
	
	static function toolTipItem($name, $description) {
		$text = Html::el('div class="tooltip-control"');
		$text->add(Html::el('span', $name));
		$text->add(
			Html::el('div class="tooltip-description"')->setHtml( 
				Html::el('div class="tooltip-description-i"')->setHtml($description)
			)
		);
		
		return $text;
	}
	
	static function getVar($name) 
	{
		return self::context()->appVars->get($name);
	}
	
	static function currentAccount()
	{
		$id = self::currentAccountId();
		return $id ? Account::get($id) : null;
		/* if (!self::context()->user->isLoggedIn()) return null;
		return self::context()->accounts->find(self::context()->user->id); */
	}
	
	static function currentAccountId() : ?int
	{
		// dump(self::context()->getService("security.user"));
		// dump(self::context()->getService("security.user")->id);
		return self::context()->getService("security.user")->id;
		// return (self::context()->user->isLoggedIn()) ? self::context()->user->id : null;
	}
	
	static function phonePrefixes() {
		return array(
			'+420' => '+420 (ČR)',
			'+421' => '+421 (SR)',
		);
	}
	
	static function phoneEreg() {
		return '(00|\+)?([0-9]\s*){9,20}';
	}
	
	static function logedUserId() {
		return self::logedUser()->id;
	}
	static function logedUser() {
		$user = self::context()->getService('security.user'); 
		return $user->isLoggedIn()
			? User::get($user->id)
			: null;
	}
	
	static function formatPrice($price) {
		// number_format($price, 0, ',', ' ')
		return \Nette\Utils\Html::el()->setHtml(
			$price === null ? '' : str_replace(' ', '&nbsp;', number_format($price, 0, ',', ' ')).'Kč'
		);
	}
	static function formatPriceSmallCurency($price) {
		return \Nette\Utils\Html::el()->setHtml(
			$price === null ? '' : str_replace(' ', '&nbsp;', number_format($price, 0, ',', ' ')).'<small>Kč</small>'
		);
	}

	/** @param null|int|string $src */
	static function utToDt($src): ?\DateTime
	{
		if ($src)
		{
			$dt = new DateTime();
			$dt->setTimestamp((int)$src);
			return $dt;
		} else return null;
	}
}

