<?php

namespace App\Presenters;

use Nette\Application\Responses\TextResponse;
use App\Product;
use App\ProductEditForm;
use App\Package;
use App\PackageEditForm;
use App\Customer;
use App\CustomerEditForm;
use App\User;
use App\UserEditForm;
use App\Prepaid;
use App\ProductSell;
use App;
use App\Glob;
use App\Receipt;
use DateTime;
use Murdej\ActiveRow\TableInfo;
use Murdej\ActiveRow\Tools\DbDeploy;
use Murdej\ProcI;
use Nette\Application\UI\Form;
use Nette\NObject;
use Nette\Utils\Paginator;
use Nette\Utils\Strings;

/**
 * Homepage presenter.
 */
class AdminPresenter extends BasePresenter
{
	public function actionDefault() 
	{
		// $this->template->data = App\App::context()->customers->fulltext($q);
	}
	
	// -- Produkty --
	public function renderProduct() 
	{
		$this->template->items = Product::getIndented();
	}
	public function createComponentProductEditForm() {
		$form = new ProductEditForm();
		$form->onClose[] = [$this, 'productCloseForm'];
		
		return $form;
	}
	public function productCloseForm() {
		$this->redirect("product");
	}
	
	public function actionProduct($editId = null, $parentId = null, $isCategory = null, $packageId = null) {
		if ($editId) {
			$this->template->itemEdit = $editId;
			$this->getComponent("productEditForm")->loadItem($editId, $parentId, $isCategory, $packageId);
		}
		$this->menu->selected = "admin:product";
	}
	public function handleProductTogle($productId, $value, $param) {
		$product = Product::get($productId);
		switch($param)
		{
			case 'enabled': $product->enabled = $value; break;
			case 'stockable': $product->stockable = $value; break;
		}
		$product->save();
		// $this->context->products->find($productId)->update(array('enabled' => $enabled));
		$this->redirect('this');
	}

	public function actionProductReorder($orders = null)
	{
		if ($orders)
		{
			$i = 1;
			foreach(explode(',', $orders) as $id)
			{
				Product::repository()->newTable()->where('id', (int)$id)->update(['position' => $i++]);
			}
		}
		else
		{
			$products = Product::findBy(['deleted' => false, 'enabled' => true]);
			$this->template->productsByPids = ProcI::from($products)->struct('(int).productId', null)->toArray();
			$this->template->productsById = ProcI::from($products)->struct('.id')->toArray();
			// dump($this->template->productsByPids);
		}
	}

	public function handleProductDelete($deleteId, $confirmed = false) {
		$product = Product::get($deleteId);
		if ($confirmed) 
		{
			$this->flashMessage('Produkt '.$product->name.' byl odstraněn.');
			$product->deleted = true;
			$product->save();
			$this->redirect('product');
		} 
		else 
		{
			$this->template->deleteProduct = $product;
		};
	}

	// -- Balicky --
	public function renderPackage() 
	{
		$this->template->items = Package::findAll();
	}
	
	public function createComponentPackageEditForm() 
	{
		$form = new PackageEditForm();
		$form->onClose[] = [$this, 'packageCloseForm'];
		
		return $form;
	}

	public function packageCloseForm() 
	{
		$this->redirect("package");
	}

	public function actionPackage($editId = null) 
	{
		$this->menu->selected = "admin:package";
	}

	// Detail
	public function actionPackageDetail($id = null) 
	{
		$this->menu->selected = "admin:package";
		$this->template->package = $id
			? Package::get($id)
			: null;
		$this->getComponent("packageEditForm")->loadItem($id);
	}

	// -- zakaznici --
	public function renderCustomer() 
	{
		$this->template->items = Customer::findAll();
	}

	public function createComponentCustomerEditForm() 
	{
		$form = new CustomerEditForm();
		$form->onClose[] = [$this, 'customerCloseForm'];
		
		return $form;
	}

	public function customerCloseForm() 
	{
		$this->redirect("customer");
	}

	public function handleCustomerDelete($deleteId, $confirmed = false) 
	{
		$customer = Customer::get($deleteId);
		if ($confirmed) {
			$this->flashMessage('Zákazník '.$customer->fullName.'Byl odstraněn.');
			$customer->deleted = true;
			$customer->save();
			$this->redirect('customer');
		} else {
			$this->template->deleteCustomer = $customer;
		};
	}
	
	public function handlePackageDelete($deleteId, $confirmed = false) {
		$package = $this->context->packages->find($deleteId);
		if ($confirmed) {
			$this->flashMessage('Pernamentka '.$package->name.' byla odstraněna ze seznamu.');
			$package->deleted = true;
			$package->update();
			$this->redirect('package');
		} else {
			$this->template->deletePackage = $package;
		};
	}
	
	public function actionCustomer($id = null) {
		if ($id) {
			$this->template->itemEdit = $id;
			$this->getComponent("customerEditForm")->loadItem($id);
		}
		$this->menu->selected = "admin:customer";
		$this->template->liveLink = $this->link("//admin:customerExport", $this->exportK);
	}

	// -- zakaznici --
	public function renderUser() 
	{
		$this->template->items = User::findAll();
	}

	public function createComponentUserEditForm() 
	{
		$form = new UserEditForm();
		$form->onClose[] = [$this, 'userCloseForm'];
		
		return $form;
	}
	public function userCloseForm() {
		$this->redirect("user");
	}
	
	public function actionUser($id = null) {
		if ($id) {
			$this->template->itemEdit = $id;
			$this->getComponent("userEditForm")->loadItem($id);
		}
		$this->menu->selected = "Admin:user";
	}
	
	// -- Sumy --
	public function actionSums($productSellId = null) {
		$this->template->data = array();
		$this->template->superadmin = \App\Glob::logedUser()->checkPerm('admin.adv-sums');
		$this->template->productSellId = $productSellId;
		if ($productSellId)
		{
			$item = ProductSell::get($productSellId);
			$this['productSellForm']->setDefaults([
				'priceDirectpaid' => $item->priceDirectpaid,
				'productId' => $item->productId,
				'utSell' => date('d.m.Y', $item->utSell),
				'utSellTime' => date('H:i:s', $item->utSell),
			]);
		}
	}
	public function createComponentSumsForm() {
		$form = new \App\DefaultForm();
		$form->addDateInput('utFrom', 'Datum od');
		$form->addDateInput('utTo', 'do');
		$form->addSelect('order', 'řadit', array("item" => 'položky', "directpay" => 'lístku'));
		$form->addCheckbox('sumsOnly', 'pouze součty');
		$form->addRadioList("stock", "Sklad", ["n" => "Ne", "y" => "Ano", "" => "Nezáleží"])->getSeparatorPrototype()->setName("span");
		$form->addSubmit('btnSubmit', 'Zobrazit');
		$form->onSuccess[] = [$this, 'sumsFormSubmited'];
		$form->setLayout('inline');
		
		return $form;
	}

	public function createComponentAdvSumsForm() {
		$form = new \App\DefaultForm();
		$form->addDateInput('utFrom', 'Datum od');
		$form->addDateInput('utTo', 'do');
		$form->addSubmit('btnSubmit', 'Zobrazit');
		$form->onSuccess[] = $this->sumsFormSubmited;
		$form->setLayout('inline');
		
		return $form;
	}

	/** @persistent */
	public $utFrom = null;
	
	/** @persistent */
	public $utTo = null;
	
	/** @persistent */
	public $order = null;
	
	/** @persistent */
	public $sumsOnly = false;

	/** @persistent */
	public $stock = null;
	
	public function sumsFormSubmited($form) {
		$utFrom = $form->values->utFrom ? $form->values->utFrom : strToTime('today');
		$utTo = $form->values->utTo ? $form->values->utTo + 86399 : strToTime('tomorrow') - 1;
		$this->utFrom = $utFrom;
		$this->utTo = $utTo;
		$this->stock = $form->values->stock;
		$this->sumsOnly = isset($form->values->sumsOnly) ? $form->values->sumsOnly : true;
		if (isset($form->values->order)) $this->order = $form->values->order;
	}
	
	public function renderSums() {
		// die('dddddffff');
		$this->template->sums = null;
		$this->template->data = null;
		if ($this->utFrom || $this->utTo) {
			// detaily
			if (!$this->sumsOnly) {
				/* $this->template->data = $this->context->productSells->findAll()
					->where('utSell BETWEEN ? AND ?', $this->utFrom, $this->utTo)
					->order(
						$this->order == 'item' ? 'productSell.utSell' : 'directpaid.utCreated'
					);*/
				$q = ProductSell::repository()->newSqlQuery()->code("SELECT
ps.id,
ps.utSell,
pr.Name AS productName,
ps.customName,
pa.name AS packageName,
ps.priceDirectpaid,
ps.priceCachless,
ps.priceCreditCA,
ps.priceCreditCL,
ps.priceCreditCA + ps.priceCreditCL AS priceCredit,
ps.productAmount,
ps.bonusCredit,
coalesce(CONCAT(cu.nameFirst, ' ', cu.nameLast), dp.note) AS customerName,
cu.id AS customerId,
ucr.name AS userCreate,
uca.name AS userCancel,
ps.cancelUt,
ps.cancelNote
FROM productSell ps
LEFT JOIN product pr ON pr.id = ps.productId
LEFT JOIN prepaid pp ON pp.id = ps.prepaidId
LEFT JOIN package pa ON pa.id = pp.packageId
LEFT JOIN directpaid dp ON dp.id = directpaidId
LEFT JOIN customer cu ON cu.id = dp.customerId
LEFT JOIN user ucr ON ucr.id = ps.createUserId
LEFT JOIN user uca ON uca.id = ps.cancelUserId
WHERE ps.deleted = 0 AND utSell BETWEEN ")->value($this->utFrom)->code(" AND ")->value($this->utTo);
			if ($this->stock) $q->c(" AND pr.stockable = ")->v($this->stock == "y");
			$q->code(" ORDER BY ".($this->order == 'item' ? 'ps.utSell' : 'dp.utCreated'));
				$this->template->data = $q->exec();
			};
			// soucty
			/* $sums['_all']['direct'] += $row->priceDirectpaid;
					$sums['_all']['credit'] += $row->priceCredit;
					$sums['_all']['amount'] += $row->productAmount;
					$groupCode = $row->productId.$row->customName;
					if (!isset($sums[$groupCode])) {
						$sums[$groupCode] = array(
							'direct' => 0, 'credit' => 0, 'amount' => 0, 'product' => $row->product->name . ' ' . $row->customName, 'count' => 0
						);
					}
					$sums[$groupCode]['direct'] += $row->priceDirectpaid;
					$sums[$groupCode]['credit'] += $row->priceCredit;
					$sums[$groupCode]['amount'] += $row->productAmount;
					$sums[$groupCode]['count']++; */
			$q = ProductSell::findAll()->selection
				->where('utSell BETWEEN ? AND ?', $this->utFrom, $this->utTo)
				->where('cancelUt IS NULL')
				->order(
					$this->order == 'item' ? 'productSell.utSell' : 'directpaid.utCreated'
				)
				// ->group(array('productId', 'customName'))
				->group('productSell.productId, customName')
				->select('SUM(priceDirectpaid) AS direct, SUM(productAmount) AS amount, SUM(priceCreditCA) AS creditCA, SUM(priceCreditCL) AS creditCL, SUM(priceCachless) AS priceCachless, COUNT(*) AS c, customName')
				->select('product.name AS productName');
				if ($this->stock) $q->where(["product.stockable" => $this->stock == "y"]);
			$this->template->sums = $q;
				// App\Glob::varbar($this->template->sums);
		}
		$this->template->utFrom = $this->utFrom;
		$this->template->utTo = $this->utTo;
		$this->template->fixScroll = "sums";
		$this->template->sumsOnly = $this->sumsOnly;
		$this->template->credit = Prepaid::getCreditInfo($this->utFrom, $this->utTo);
	}
	


	public function actionAdvsums() {
		// die('dddddffff');
		$this->template->sums = null;
		if ($this->utFrom || $this->utTo) {
			$sums = array();
			/* krpuje
			foreach($this->context->productSells->findAll(true, false)
				->where('utSell BETWEEN ? AND ?', $this->utFrom, $this->utTo)
				->where('priceDirectpaid IS NOT NULL')
				->group(array('productId', 'deleted'))
				->select('SUM(priceDirectpaid) AS p, COUNT(*) AS c, productId, deleted') as $row // 
			) { */
			foreach(\App\ProductSell::repository()->newSqlQuery()
				->c("SELECT SUM(priceDirectpaid) AS p, COUNT(*) AS c, productId, deleted "
					."FROM productSell "
					."WHERE utSell BETWEEN $this->utFrom AND $this->utTo AND priceDirectpaid IS NOT NULL AND `cancelUt` IS NULL "
					."GROUP BY productId, deleted ")
				->getSelection() as $row	
			) {
				if (!isset($sums[$row->productId])) {
					$sums[$row->productId] = new AdvSumRow($row->productId);
				}
				$r = $sums[$row->productId];
				if ($row->deleted) {
					$r->countD += $row->c;
					$r->priceD += $row->p;
				} else {
					$r->countV += $row->c;
					$r->priceV += $row->p;
				}
			}
			$this->template->sums = $sums;

			$form = $this->getComponent("advSumsFormChange");
			$form->loadData($sums, $this->utFrom, $this->utTo);
		}
		$this->template->utFrom = $this->utFrom;
		$this->template->utTo = $this->utTo;
		$this->template->fixScroll = "advSums";
		
	}

	public function createComponentProductSellForm()
	{
		$form = new \App\DefaultForm();
		$form->addSelect('productId', 'Produkt', Product::findAll()->fetchPairs('id', 'name'));
		$form->addText('priceDirectpaid', 'Cena hotově')->setType("number")
			->addRule($form::FLOAT, 'Cena musí být číslo')
			->setRequired(false);
		$form->addDateInput('utSell', 'Datum')
			->setRequired('Zadejte datum');
		$form->addText('utSellTime', 'Čas')
			->addRule($form::PATTERN, 'Chybný formát času', '^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$')
			->setRequired('Zadejte čas');
		$form->addSubmit('btSave', 'Uložit');
		$form->onSuccess[] = [ $this, 'saveProductSell' ];

		return $form;
	}

	public function saveProductSell($form, $values)
	{
		$id = $this->getParameter('productSellId');
		$item = ProductSell::get($id);
		$item->productId = $values->productId;
		$item->priceDirectpaid = $values->priceDirectpaid;
		list($h, $m, $s) = explode(':', $values->utSellTime);
		$item->utSell = $values->utSell + $h * 3600 + $m * 60 + $s;

		$item->save();
		$this->flashMessage("Uloženo");
		$this->redirect('this', ['productSellId' => null]);
	}
	
	public function createComponentAdvSumsFormChange() {
		$form = new App\AdvSumFormChange();
		$form->onChange[] = $this->AdvSumsChange;
		return $form;
	}
	
	public function AdvSumsChange($res) {
		App\Glob::varbar($res);
		foreach($res as $productId => $diff) {
			/* $ids = $this->context->productSells->findAll(false, false)
				->where('productId', $productId)
				->where('utSell BETWEEN ? AND ?', $this->utFrom, $this->utTo)
				->where('priceDirectpaid IS NOT NULL')
				->where('deleted', ($diff > 0))
				->order('RAND()')
				->limit(abs($diff))
				->fetchPairs("id", "id"); */
			$ids = array_values(ProductSell::repository()->newSqlQuery()
				->c(
					"SELECT id, id FROM productSell "
					."WHERE productId = $productId AND utSell BETWEEN $this->utFrom AND $this->utTo AND priceDirectpaid IS NOT NULL "
					."AND deleted = ".($diff > 0 ? "1" : "0")." AND `cancelUt` IS NULL "
					."ORDER BY RAND() LIMIT ".abs($diff)
				)->fetchPairs("id", "id"));
			if ($ids) {
				ProductSell::repository()->newSqlQuery()->c(
					"UPDATE productSell "
					."SET deleted = " . ($diff < 0 ? "1" : "0") . " "
					."WHERE id IN (" . implode(", ", $ids) . ") "
				)->exec();
			}
			
			App\Glob::varbar($ids);
		}
		$this->redirect("this");
	}

	public function handleDeleteProductSell($id)
	{
		$ps = ProductSell::get($id);
		$ps->deleted = true;
		$ps->save();
	}
	
	// -- import --
	public function createComponentInitImportForm() {
		$form = new App\DefaultForm();
		$form->addTextarea('data', 'Data');
		$form->addSelect('mode', 'Režim', array('product' => 'Vstupy', 'credit' => 'Kredit'));
		$form->addCheckbox('insert', 'Provest');
		$form->addSubmit('btnSubmit', 'Importovat');
		$form->onSuccess[] = $this->initImportFormSubmited;
		$form->setLayout('inline');
		
		return $form;
	}
	public function initImportFormSubmited($form) {
		$exec = $form->values->insert;
		$mode = $form->values->mode;
		$res = array();
		foreach(explode("\n", $form->values->data) as $line) {
			$line = trim($line);
			if ($line) {
				$res[] = $line;
				list($nameFirst, $nameLast, $prepaidName, $usedProducts) = explode("\t", $line);
				// Najdi zakaznika
				$customer = $this->context->customers->findOneBy(array(
					'nameFirst' => $nameFirst,
					'nameLast' => $nameLast,
				));
				App\Glob::varbar($customer);
				if (!$customer) {
					// Novy zakaznik
					if ($exec) {
						$customer = $this->context->customers->insert(array(
							'nameFirst' => $nameFirst,
							'nameLast' => $nameLast,
						));
					}
					$res[] = ' - Pridavam zakaznika';
				}
				// Nova pernamentka
				$package = $this->context->packages->findOneBy('name', $prepaidName);
				if (!$package) {
					$res[] = ' - Nenalezena pernametka "'.$prepaidName.'"';
				} else {
					if ($exec) {
						$prepaid = $package->makePrepaid($customer->id);
						switch($mode) {
							case 'product':
								$ppp = $prepaid->related('prepaidPackageProduct')->fetch();
								$ppp->amountLeft -= $usedProducts;
								$ppp->update();
								break;
							case 'credit':
								$prepaid->creditLeft = $usedProducts;
								$prepaid->update();
								break;
						}
					}
					$res[] = ' - Ok';
				}
			}
		}
		$this->template->log = $res;
	}
	public function actionExport() {
		$response = $this->context->httpResponse;
		$response->setContentType('text/plain', 'UTF-8');
		$response->setHeader('Content-Disposition', 'attachment; filename=fitko-zaloha-'.date('Y-m-d_H-i-s').'.dbdump');
		//$response->setHeader('Content-Transfer-Encoding', 'binary');
		//$response->setHeader('Expires', 0);
		//$response->setHeader('Cache-Control', 'must-revalidate, post-check=0, pre-check=0');
		//$response->setHeader('Pragma', 'public');
		$csv = $this->context->dbDump->dump();
		$this->sendResponse(new TextResponse($csv->buffer));
		$this->terminate();
	}
	
	public function handleAddNote($productSellId) {
		$this->template->addNoteProductSellId = $productSellId;
		$this->getComponent("formAddNote")->setDefaults(array('productSellId' => $productSellId));
	}
	
	public function createComponentFormAddNote() {
		$form = new App\DefaultForm();
		$form->addTextarea('note', 'Poznámka');
		$form->addHidden('productSellId');
		$form->addSubmit('btnSubmit', 'Uložit');
		$form->onSuccess[] = $this->formAddNoteSubmited;
		
		return $form;
	}
	
	public function formAddNoteSubmited($form) {
		$this->context->productSellNotes->insert(array(
			'productSellId' => $form->values['productSellId'],
			'note' => $form->values['note'],
			'utAdd' => time(),
			'userId' => $this->context->user->id
		));
		$this->flashMessage('Poznámka přidána.');
	}
	
	public function handleUserDelete($deleteId, $confirmed = false) {
		$user = $this->context->users->find($deleteId);
		if ($confirmed) {
			$this->flashMessage('Uživatel'.$user->name.'Byl odstraněn.');
			$user->deleted = true;
			$user->update();
			$this->redirect('user');
		} else {
			$this->template->deleteUser = $user;
		};
	}

	public function actionDbDeploy($tables)
	{
		$this->template->sql = "";
	}

	public function createComponentDbDeployForm() 
	{
		global $_robotloader;
		$form = new App\DefaultForm();
		$tis = ProcI::from($_robotloader->getIndexedClasses())
			->filter(function($_, $cn) { return Strings::startsWith($cn, "App\\"); })
			->map(function($_, $cn) { return new TableInfo($cn); })
			->filter(function($ti) { return $ti->tableName; })
			->orderBy(".className")
			->struct(".className");
		$form->addCheckboxList("classes", "Modely", ProcI::from($tis)->map(".className")->toArray());
		$form->addSubmit("bShowSQL", "Ukaž SQL");
		$form->onSuccess[] = function ($form, $values) use ($tis)
		{
			$dbd = new DbDeploy();
			$this->template->sql = $dbd->createTables($tis
				->filter(function($ti) use ($values) { return in_array($ti->className, $values->classes); })
				->toArray()
			);
		};
			
		return $form;
	}

	// Účtenky
	/** @persistent */
	public $filter = [];

	public function renderReceipts(int $page = 1)
	{
		$conds = [];
		$fo = (object)($this->filter + ["dtFrom" => null, "dtTo" => null, "states" => null]);
		if ($fo->dtFrom) $conds["dtCreate >= ?"] = Glob::utToDt($fo->dtFrom);
		if ($fo->dtTo) $conds["dtCreate <= ?"] = Glob::utToDt($fo->dtTo);
		if ($fo->states) $conds["state"] = $fo->states;

		$paginator = new Paginator();
		$paginator->itemsPerPage = 10;
		$paginator->itemCount = Receipt::findBy($conds)->count();
		$paginator->page = $page;

		$this->template->receipts = Receipt::findBy($conds)->limit($paginator->getLength(), $paginator->getOffset());
		$this->template->paginator = $paginator;
	}

	public function createComponentReceiptFilter()
	{
		$form = new Form();
		$form->setMethod("get");
		$form->addDateInput("dtFrom", "Datum a čas od");
		$form->addDateInput("dtTo", "Datum a čas do");
		$form->addCheckboxList("states", "Stav", Receipt::stateNames);
		$form->addSubmit("btShow", "Zobrazit");
		$form->addHidden("page", 1);
		$form->onSuccess[] = function($form, $values)
		{
			$this->filter = ProcI::from($values)->toArray();
		};
		$form->setDefaults($this->filter);
		
		return $form;
	}

	public function renderPhpInfo()
	{
		phpinfo();
		exit();
	}

	protected $exportK = "xc1y5bYZ85b5x58Qg6x5c49d8g63c20";

	public function renderCustomerExport($k)
	{
		if ($k != $this->exportK) return "";
		$this->template->colDefs = [
			"id" => [ "label" => "ID" ],
			"nameFirst" => [ "label" => "Jméno" ],
			"nameLast" => [ "label" => "Příjmení" ],
			"note" => [ "label" => "Poznámka" ],
			"code" => [ "label" => "Kód" ],
			"contactPhone" => [ "label" => "Telefon" ],
			"contactAddress" => [ "label" => "Adresa" ],
			"dtBirth" => [ "label" => "Datum narození" ],
			"rc" => [ "label" => "Rodné číslo" ],
			"age" => [ "label" => "Věk" ],
			"utLast" => [ "label" => "Datum posledního pohybu" ],
		];
		$rows = [];
		list($cy, $cmd) = $this->dt2z_md(new DateTime());
		$dtFormat = "Y-m-d\\TH:i:s"; // "d.m.Y H:i:s";
		$dtFormatB = "Y-m-d"; // "d.m.Y";
		// foreach(Customer::findAll(["deleted" => true]) as $customer)
		foreach(Customer::repository()->newSqlQuery()->code(
			"SELECT c.*, MAX(dp.utCreated) AS utLast
			FROM customer c
			LEFT JOIN directpaid dp ON dp.customerId = c.id
			GROUP BY c.id
			-- LIMIT 50 "
		)->exec() as $row/*$customer*/)
		{
			// dump($row);
			// $row = $customer->toArray();
			if ($row["dtBirth"])
			{
				list($ry, $rmd) = $this->dt2z_md($row["dtBirth"]);
				$row["age"] = $cy - $ry - ($rmd > $cmd ? 1 : 0);
				$row["dtBirth"] = $row["dtBirth"]->format($dtFormatB);
			}
			$row["utLast"] = $row["utLast"] ? date($dtFormat, $row["utLast"]) : null;

			$rows[] = $row;
		}
		$this->template->rows = $rows;
	}

	public function dt2z_md(\DateTime $dt) 
	{
		return explode("-", $dt->format("Y-md"));
	}
}

class AdvSumRow extends NObject {
	public $countV = 0;
	
	public $priceV = 0;
	
	public $countD = 0;
	
	public $priceD = 0;
	
	public $productId;
	
	public function __construct($productId) {
		$this->productId = $productId;
	}
	public function getProduct() {
		return Product::get($this->productId);
	}
	public function getCountS() {
		return $this->countV + $this->countD;
	}
}
