<?php

namespace App;

use Murdej\Duration;

class ComPackage extends DefaultComponent {
	/** @persistent */
	public $validModify = false;

	/** @persistent */
	public $changeCredit = false;

    /** @persistent */
    public $prepaidId;

    public function render($par) {
		$prepaidId = null;
		if ($par[0] == 'a') { // package
			$packageId = (int)\substr($par, 1);
		} else {
			$prepaidId = (int)\substr($par, 1);
		}
		if ($prepaidId) {
			$this->template->prepaid = Prepaid::get($prepaidId);
			$packageId = $this->template->prepaid->packageId;
			Glob::varbar($packageId);
		} else {
			$this->template->prepaid = null;
		}
		$this->template->package = Package::get($packageId);
		$this->template->showActivate = $this->template->prepaid && $this->template->prepaid->instore;
		$this->template->validModify = $this->validModify;
		$this->template->changeCredit = $this->changeCredit;

		$this->baseRender();
	}
	public function handleValidModify($prepaidId) {
		$this->validModify = true;
        $this->prepaidId = $prepaidId;
		$form = $this->getComponent('validForm');
		$form->setDefaults(Prepaid::get($prepaidId)->toArray());
	}

	public function createComponentValidForm() {
		$form = new DefaultForm();
        $prepaid = $this->prepaidId ? Prepaid::get($this->prepaidId) : null;
		$form->addHidden('id');
		$form->addDateInput('utValidFrom', 'Platnost od')->setRequired('Zadejte platnost od');
		$form->addDateInput('utValidTo', 'Platnost dd')->setRequired('Zadejte platnost do');
		$form->addTextArea('note', 'Důvod změny')->setRequired('Zadejte důvod změny');
		$form->addSubmit('btnSave', 'Nastavit platnost')->onClick[] = [$this, 'validSave'];

        $status = $prepaid ? $prepaid->getStatus() : null;

        if (!$status || $status != Prepaid::State_invalid)
            $form->addSubmit('btnAbort', 'Předčasně ukončit')
                ->onClick[] = [$this, 'validAbort'];
        if (!$status || $status != Prepaid::State_instore)
            $form->addSubmit('btnToNew', 'Zpět na neaktivní')
                ->onClick[] = [$this, 'validToNew'];
        if (!$status || $status == Prepaid::State_active)
            $form->addSubmit('btnSuspend', 'Pozastavit')
                ->onClick[] = [$this, 'suspend'];
        if (!$status || $status == Prepaid::State_suspended)
            $form->addSubmit('btnRestore', 'Obnovit')
                ->onClick[] = [$this, 'resume'];
		$form->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'validCancel'];
			
		return $form;
	}
	public function startup() {
	}
	
	public function validSave($button) {
		$values = $button->form->values;
		$prepaid = Prepaid::get($values->id);
		/*Glob::context()->prepaidChanges->insert(array(
			'change' => "Změna platnosti pernametky z "
				.Glob::formatDate($prepaid->utValidFrom).' - '.Glob::formatDate($prepaid->utValidTo)
				.' na '
				.Glob::formatDate($values->utValidFrom).' - '.Glob::formatDate($values->utValidTo)
				.'.',
			'prepaidId' => $values->id,
			'note' => $values->note
		));*/
        $prepaid->beforeChange();
        $prepaid->utValidFrom = $values->utValidFrom;
        $prepaid->utValidTo = $values->utValidTo;
        $prepaid->save();
		$prepaid->addChange(
			'Změna platnosti pernametky z '
			.Glob::formatDate($prepaid->utValidFrom).' - '.Glob::formatDate($prepaid->utValidTo)
			.' na '
			.Glob::formatDate($values->utValidFrom).' - '.Glob::formatDate($values->utValidTo)
			.'.',
			$values->note
		);
		$this->presenter->flashMessage("Platnost pernamentky byla změněna");
		$this->closeValidModify();
	}

    protected function closeValidModify()
    {
        $this->validModify = false;
        $this->redirect("this");
    }

	public function validToNew($button) 
	{
		$values = $button->form->values;
		$prepaid = Prepaid::get($values->id);
        $prepaid->beforeChange();
		$prepaid->utValidFrom = null;
		$prepaid->utValidTo = null;
        $prepaid->suspendValidLeft = null;
		$prepaid->addChange(
			'Pernamentka byla převedena zpět do stavu neaktivní',
			$values->note
		);
		$prepaid->save();
		$this->presenter->flashMessage("Platnost pernamentky byla změněna");
        $this->closeValidModify();
	}

    public function suspend($button)
    {
        $values = $button->form->values;
        $prepaid = Prepaid::get($values->id);
        $prepaid->beforeChange();
        $prepaid->suspendValidLeft = $prepaid->utValidTo - time();
        $prepaid->utValidTo = time();
        $prepaid->addChange(
            'Platnost pernamentky byla pozastavena. Zbývá '.Duration::toString($prepaid->suspendValidLeft),
            $values->note
        );
        $prepaid->save();
        $this->presenter->flashMessage("Platnost pernamentky byla pozastavena");
        $this->closeValidModify();
    }

    public function resume($button)
    {
        $values = $button->form->values;
        $prepaid = Prepaid::get($values->id);
        $prepaid->beforeChange();
        $prepaid->utValidTo = time() + $prepaid->suspendValidLeft;
        $prepaid->suspendValidLeft = null;
        $prepaid->addChange(
            'Platnost pernamentky byla pozastavena. Zbývá '.Duration::toString($prepaid->suspendValidLeft),
            $values->note
        );
        $prepaid->save();
        $this->presenter->flashMessage("Platnost pernamentky byla pozastavena");
        $this->closeValidModify();
    }

    public function validAbort($button)
	{
		$values = $button->form->values;
		$prepaid = Prepaid::get($values->id);
        $prepaid->beforeChange();
		$prepaid->utValidTo = time();
        $prepaid->suspendValidLeft = null;
		$prepaid->addChange(
			'Pernamentka byla předčasně ukončena',
			$values->note
		);
		$prepaid->save();
		$this->presenter->flashMessage("Platnost pernamentky byla změněna");
        $this->closeValidModify();
	}

	public function validCancel($form) 
	{
		$this->validModify = false;
	}

	public function createComponentChangeCreditForm() 
	{
		$form = new DefaultForm();
		$form->addHidden('id');
		$form->addText("amount", "Hotovost")->setType("number")
			->setRequired("Zadejte částku")
			->addRule($form::FLOAT, 'Částka musí být číslo');
		/* $form->addText("bonusCredit", "Bonus kredit / bezhotovostně")->setType("number")
			->setRequired(false)
			->addCondition($form::FILLED)
				->addRule($form::FLOAT, 'Částka musí být číslo'); */
		$form->addText("priceCachless", "Bezhotovostně")->setType("number")
			->setRequired(false)
			->addCondition($form::FILLED)
				->addRule($form::FLOAT, 'Částka musí být číslo');
		$form->addTextArea("note", "Poznámka");
		// $form->addSubmit('btnComputeBonus', 'Navrhnout bonus')->onClick[] = [$this, 'computeBonus'];
		$form->addSubmit('btnSave', 'Dobít kredit')->onClick[] = [$this, 'changeCreditSave'];
		$form->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'changeCreditCancel'];
			
		return $form;
	}

	public function handleChangeCredit($prepaidId) 
	{
		$this->changeCredit = true;
		$form = $this->getComponent('changeCreditForm');
		$form->setDefaults(Prepaid::get($prepaidId)->toArray());
	}

	public function changeCreditCancel($button) 
	{
		$this->presenter->flashMessage("Dobití kreditu bylo zrušeno");
		$this->changeCredit = false;
	}

	public function changeCreditSave($button) 
	{
		$this->presenter->flashMessage("Dobití kreditu bylo zaznamenáno a bude provedeno po uzavření tohoto lístku");
		$values = $button->form->values;
		$pp = new ProductPrice('changeCredit-'.$values->id.'-'.$values->amount);
		$pp->bonusCredit = $values->bonusCredit;
		$pp->priceCachless = $values->priceCachless;
		$directpaid = Directpaid::byCustomer(Prepaid::get($values->id)->customerId);
		$this->flashMessage($directpaid->add($pp, $values->note));
		$this->changeCredit = false;
		$this->redirect('this');
	}

	public function computeBonus($button) 
	{
		$amount = $button->form->values->amount;
		$per = Glob::context()->getService('bonusCreditCalculator')->compute($amount);
		$bonusCredit = round($amount * ($per / 100));
		$button->form['bonusCredit']->setValue($bonusCredit);
		$this->presenter->flashMessage($per ? "Bonus je $per% : $bonusCredit" : "Bez nároku na bonus");
	}
}