<?php

namespace Murdej;

class Duration {
	const secY = 31557600;
	
	const secM = 2629800;
	
	const secD = 86400;
	
	static function toString($sec) {
		list($y, $m, $d) = self::secToYMD($sec);
		
		$str = ($y ? $y.'r ' : '').($m ? $m.'m ' : '').($d ? $d.'d' : '');
		return $str ? $str : "0d";
	}

	static function toLongString($sec, $blankStr = "nenastaveno") {
		list($y, $m, $d) = self::secToYMD($sec);
		
		$str = 
			($y ? self::plural('y', $y).' ' : '')
			.($m ? self::plural('m', $m).' ' : '')
			.($d ? self::plural('d', $d).' ' : '');
		return $str ? ($sec < 0 ? 'před ' : '').$str : $blankStr;
	}
	
	static function plural($word, $num) {
		$neg = $num < 0;
		$num = abs($num);
		if ($num == 1) {
			$n = 0;
		} else if (($num < 5) && ($num > 1)) {
			$n = 1;
		} else {
			$n = 2;
		}
		if ($neg) $n += 3;
		return $num.' '.self::$pluralData[$word][$n];
	}
	
	static $pluralData = array(
		'y' => array('rok', 'roky', 'let', 'rokem', 'roky', 'lety'),
		'm' => array('měsíc', 'měsíce', 'měsíců', 'měsícem', 'měsíci'),
		'd' => array('den', 'dny', 'dnů', 'dnem', 'dny', 'dny'),
	);
	
	static function secToYMD($sec) {
		$neg = $sec < 0;
		$sec = abs($sec);
		// Roky
		$y = floor($sec / self::secY);
		$sec = $sec % self::secY;
		// Mesice
		$m = floor($sec / self::secM);
		$sec = $sec % self::secM;
		// Dny
		$d = floor($sec / self::secD);
		
		if ($neg) {
			$y *= -1;
			$m *= -1;
			$d *= -1;
		}
		return array($y, $m, $d);
	}

	static function parse($str) {
		$sec = 0;
		foreach(explode(' ', $str) as $chunk) if (trim($chunk)) {
			$chunk = strtolower(trim($chunk));
			$u = substr($chunk, -1);
			$c = (int)$chunk;
			switch($u) {
				case 'd': $sec += $c * self::secD; break;
				case 'm': $sec += $c * self::secM; break;
				case 'r': $sec += $c * self::secY; break;
			}
		}
		
		return $sec;
	}
}
