<?php

namespace Murdej\Forms;

use App\Model\Utils;
use Nette;
use Nette\Forms\Container as FormContainer;

trait TMultiChoiceControlMissingValueCallback
{
	/**
	 * @param null|function($values) : array
	 */
	public $missingValueCallback = null;

	/**
	 * Sets selected items (by keys).
	 * @param  array
	 * @return static
	 * @internal
	 */
	public function setValue($values)
	{
		if (is_scalar($values) || $values === null) {
			$values = (array) $values;
		} elseif (!is_array($values)) {
			throw new Nette\InvalidArgumentException(sprintf("Value must be array or null, %s given in field '%s'.", gettype($values), $this->name));
		}
		$flip = [];
		foreach ($values as $value) {
			if (!is_scalar($value) && !method_exists($value, '__toString')) {
				throw new Nette\InvalidArgumentException(sprintf("Values must be scalar, %s given in field '%s'.", gettype($value), $this->name));
			}
			$flip[(string) $value] = true;
		}
		$values = array_keys($flip);
		if ($diff = array_diff($values, array_keys($this->items)))
		{
			if ($this->missingValueCallback)
			{
				$cb = $this->missingValueCallback;
				$this->items += $cb($diff);
			}
			if ($this->checkAllowedValues && ($diff = array_diff($values, array_keys($this->items))))
			{
				$set = Nette\Utils\Strings::truncate(implode(', ', array_map(function ($s) { return var_export($s, true); }, array_keys($this->items))), 70, '...');
				$vals = (count($diff) > 1 ? 's' : '') . " '" . implode("', '", $diff) . "'";
				throw new Nette\InvalidArgumentException("Value$vals are out of allowed set [$set] in field '{$this->name}'.");
			}
		}
		$this->value = $values;
		return $this;
	}

	/**
	 * Returns selected keys.
	 * @return array
	 */
	public function getValue(): array
	{
		if ($diff = array_diff($this->value, array_keys($this->items)))
		{
			if ($this->missingValueCallback)
			{
				$cb = $this->missingValueCallback;
				$this->items += $cb($diff);
			}
		}
		$values = array_values(array_intersect($this->value ?: [], array_keys($this->items)));
		return $values;
	}

	public static function register()
	{
		$cn = get_called_class();
		FormContainer::extensionMethod(
			'add'. Utils::classNameNNS($cn), 
			function($form, $name, ...$args) use ($cn)
			{
				return $form[$name] = new $cn(...$args);
			}
		);
	}
}
